@@ -11,11 +11,11 @@ __all__ = [
 import unittest
 import doctest
 import re
+import time
 
 import gevent
 
-class WaitTimeout:
-    pass
+from xnet.net.ipv4 import AddressIterator
 
 
 class ToolException(Exception):
@@ -112,6 +112,7 @@ class ToolError:
         0: 'CustomError',
         1: 'ParseFailed',
         2: 'ActionFailed',
+        3: 'WaitTimeout',
     }
     CustomError = 0
     ParseFailed = 1
@@ -129,6 +130,10 @@ class ToolError:
         return self.message
 
 
+class WaitTimeout:
+    pass
+
+
 class Tool(object):
     '''
         Must implement __action__, all other methods are optional.
@@ -141,7 +146,15 @@ class Tool(object):
         that action value is not available.
 
         @classmethod
-        __massage__(iterator)
+        __setup__(options)
+            input: commandline options object
+            output: None
+
+                Prepare class and verify command-line options, before
+                input parsing and spawning instances of class.
+
+        @classmethod
+        __massage__(iterator, options)
             input: iterator with all input
             output: generate (yield) items to __parse__()
 
@@ -195,6 +208,11 @@ class Tool(object):
                 Default __timeout__() returns None.
                 May set self.stderr to True to force output to stderr.
 
+        __filename__()
+            output: str
+
+                Used to produce file paths for --split-output and --tee-output
+
 
     '''
     PRE_CALL = 0
@@ -210,6 +228,8 @@ class Tool(object):
         self._action_result = None
         self._state = self.PRE_CALL
         self._error = None
+        self._action_starttime = None
+        self._action_endtime = None
         self._cleanup_funcs = []
         self.stderr = False  # if True, print output to stderr
         self._validate_implementations()
@@ -245,8 +265,8 @@ class Tool(object):
                 #
                 # Do some handling, if Timeout works...
                 #
-            except WaitTimeout, e:
-                self.set_error('wait-timeout')
+            except WaitTimeout:
+                self.set_error(WaitTimeout)
             finally:
                 if timeout:
                     timeout.cancel()
@@ -284,10 +304,12 @@ class Tool(object):
         #
         # __action__
         #
+        self._action_starttime = time.time()
         action_result = self.__action__(parse_result)
+        self._action_endtime = time.time()
         if action_result is None or not self.error is None:
             if self.error is None:
-                print 'kk IT IS NONE', parse_result
+                print 'kk error IS NONE', parse_result
                 code = ToolError.ActionFailed
                 error = ToolError(code)
                 self.set_error(error)
@@ -295,6 +317,13 @@ class Tool(object):
         if not isinstance(action_result, dict):
             msg = '__action__() did not return a dict subclass.'
             raise InvalidToolResult(msg)
+
+        #
+        # Some action_results should always be available:
+        #
+        t = 1000 * (self._action_endtime - self._action_starttime)
+        action_result['time'] = '%.1f' % t
+
         #
         # __filter__
         #
@@ -330,6 +359,12 @@ class Tool(object):
         value = self._action_result
         return value
 
+    @property
+    def action_time(self):
+        if None in (self._action_starttime, self._action_endtime):
+            return None
+        return self._action_endtime - self._action_starttime
+
     @property
     def error(self):
         return self._error
@@ -395,11 +430,19 @@ class Tool(object):
         assert self._state in (self.PRE_CALL, self.POST_CALL)
         return self._state == self.POST_CALL
 
+    #
+    # __setup__ optional preparations before we start parsing input and spawning
+    # instances.
+    #
+    @classmethod
+    def __setup__(cls, options):
+        pass
+
     #
     # __massage__ must return iterator of items
     #
     @classmethod
-    def __massage__(self, iterator):
+    def __massage__(self, iterator, options):
         return iterator
 
     #
@@ -440,8 +483,55 @@ class Tool(object):
     def __format_help__(cls):
         return 'Not implemented'
 
+    #
+    # Should generally be implemented by Tools
+    # to avoid unexpected behaviour.
+    #
+    def __filename__(self):
+        name = str(self._item).strip()
+        name = name.replace('/', '__')
+        name = name.replace('\\', '--')
+        return name
 
 
+class ToolMixin_AddrIterator(object):
+    '''
+        This iterator handles hosts and IP-ranges,
+        with optional port ranges, such as:
+            10.0.0.0/24
+            10.0.0.0/255.255.255.0
+            10.0.*.5-8
+            192.168.0.5:445
+            192.168.0.5:22,23,22-100
+            192.168.0.0/28:22,23,22-100
+            172.20.1-3.*:1-1024,8080
+
+        And argument-specified ports such as:
+            -p 3306
+            -p 22,23,25
+            -p 22-100
+            -p 80,443,8080-8089
+
+        Note: hosts with explicitly specified ports (such as
+        192.168.0.1:445) will never be yielded with any other
+        ports than 445. Colon-specified port ranges override
+        argument-specified ranges.
+    '''
+    @classmethod
+    def __massage__(cls, iterator, options):
+        ai = AddressIterator(options.port or '80')
+        for xaddr in iterator:
+            for addr in ai.expand(xaddr):
+                yield addr
+
+    def __parse__(self, item, iterator):
+        host_port = item.strip()
+        (host, port) = host_port.split(':')
+        addr = (host, int(port))
+        self._addr = addr  # for __timeout__()
+        result = {'addr': addr}
+        return result
+
 class Test1(unittest.TestCase):
     def test_file(self):
         pass
