@@ -21,14 +21,22 @@ class IISShortnameTool(WebgetTool):
     __itemname__ = 'url'
 
     cmdline_options = [
+        ('-p', '--ports', 'use these ports for urls lacking port AND protocol components',
+            dict(dest='ports', metavar='m-n,..')),
         ('-s', '--server', 'print server header',
             dict(dest='server', action='store_true')),
         ('-S', '--https', 'default to https when protocol is missing',
             dict(dest='https', action='store_true')),
         ('', '--host', 'specify Host header manually',
             dict(dest='host')),
-        ('', '--proxy', 'use this proxy for all protocols',
+        ('', '--method', 'specify alternative method. Examples GET, HEAD, POST, ...',
+            dict(dest='method', metavar='method')),
+        ('', '--proxy', 'not working, sorry!',
             dict(dest='proxy')),
+        ('', '--redirect-allow-samedomain', 'follow redirects under the same domain',
+            dict(dest='redirect_allow_samedomain', action='store_true')),
+        ('', '--redirect-whitelist', 'comma-separated whitelist of allowed redirect hosts',
+            dict(dest='redirect_whitelist', metavar='hosts')),
         ('', '--code', 'only print response code',
             dict(dest='code', action='store_true')),
     ]
@@ -41,17 +49,17 @@ class IISShortnameTool(WebgetTool):
         super(IISShortnameTool, self).__init__(*args, **kw)
         self._url = None
 
-    @classmethod
-    def __massage__(self, iterator):
-        for item in iterator:
-            item = item.strip()
-            try:
-                iprangeiter = IPRangeIterator(item)
-            except ValueError:
-                yield item
-            else:
-                for ip in iprangeiter:
-                    yield str(ip)
+    #@classmethod
+    #def __massage__(self, iterator, options):
+    #    for item in iterator:
+    #        item = item.strip()
+    #        try:
+    #            iprangeiter = IPRangeIterator(item)
+    #        except ValueError:
+    #            yield item
+    #        else:
+    #            for ip in iprangeiter:
+    #                yield str(ip)
 
     def __parse__(self, line, iterator):
         result = super(IISShortnameTool, self).__parse__(line, iterator)
@@ -133,6 +141,7 @@ Format variables for %s:
  vulnerable  - boolean result
  code1       - response code from url+'*~1*/.aspx'
  code2       - response code from url+'poipoipoi*~1*/.aspx'
+ time        - time consumed by action, in ms
 
 Default format:
  '{url} {host} {vulnerable} {code1} {code2}'
