@@ -7,16 +7,13 @@
 __all__ = ['TcpStateTool']
 
 
-import unittest
-import doctest
 import socket
-import gevent
 
 from xnet.tools import Tool
-from xnet.net.ipv4 import IPRangeIterator
+from xnet.tools import ToolMixin_AddrIterator
 
 
-class TcpStateTool(Tool):
+class TcpStateTool(ToolMixin_AddrIterator, Tool):
     '''
         TcpStateTool
             Default format is:
@@ -30,11 +27,13 @@ class TcpStateTool(Tool):
     __itemname__ = 'host'
     __description__ = 'test if TCP-ports are open'
 
+    DEFAULT_PORT = 80
+
     cmdline_options = [
         ('-p', '--port', 'port to state check',
             dict(dest='port')),
-        ('-O', '--open', 'print only results for open ports',
-            dict(dest='open')),
+        ('-O', '--open', 'only print open ports',
+            dict(dest='open', action='store_true')),
     ]
 
     @classmethod
@@ -48,28 +47,6 @@ class TcpStateTool(Tool):
             return '<tcpstate {0}:{1} {2}>'.format(host, port, state)
         return '<tcpstate {0}>'.format(state)
 
-    @classmethod
-    def __massage__(self, iterator):
-        for item in iterator:
-            item = item.strip()
-            try:
-                iprangeiter = IPRangeIterator(item)
-            except ValueError:
-                yield item
-            else:
-                for ip in iprangeiter:
-                    yield str(ip)
-
-    def __parse__(self, item, iterator):
-        port = 80
-        if not self.options.port is None:
-            port = int(self.options.port)
-        host = item.strip()
-        addr = (host, port)
-        self._addr = addr  # for __timeout__()
-        result = {'addr': addr}
-        return result
-
     def __action__(self, parse_result):
         addr = parse_result['addr']
         state = None
@@ -104,14 +81,25 @@ class TcpStateTool(Tool):
         result['state'] = state
         return result
 
+    def __filter__(self, action_result):
+        if self.options.open:
+            return action_result and action_result['state'] == 'open'
+        return action_result
+
     def __format__(self, line, parse_result, action_result):
         output = ''
         if action_result:
-            output = '{0} {1} {2}'.format(
-                action_result['host'],
-                action_result['port'],
-                action_result['state'],
-            )
+            if self.options.open:
+                output = '{0} {1}'.format(
+                    action_result['host'],
+                    action_result['port'],
+                )
+            else:
+                output = '{0} {1} {2}'.format(
+                    action_result['host'],
+                    action_result['port'],
+                    action_result['state'],
+                )
         return output
 
     @classmethod
@@ -122,6 +110,7 @@ Format variables for %s:
  host    - target host
  port    - target port
  state   - TCP-state or error-message
+ time    - time consumed by action, in ms
 
 Default format:
  '{host} {port} {state}'
@@ -131,7 +120,7 @@ Default format:
     def __timeout__(self):
         if hasattr(self, '_addr'):
             host, port = self._addr
-            return '{0} {1} timeout\n'.format(host, port)
+            return '{0} {1} wait-timeout\n'.format(host, port)
         return None
 
 
